@@ -1,6 +1,6 @@
 /* Disassemble support for GDB.
 
-   Copyright (C) 2000-2024 Free Software Foundation, Inc.
+   Copyright (C) 2000-2025 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -28,7 +28,6 @@
 #include "cli/cli-cmds.h"
 #include "dis-asm.h"
 #include "source.h"
-#include "gdbsupport/gdb-safe-ctype.h"
 #include <algorithm>
 #include <optional>
 #include "valprint.h"
@@ -595,7 +594,7 @@ do_mixed_source_and_assembly_deprecated
     {
       mle[newlines].line = le[i].line;
       mle[newlines].start_pc = le[i].pc (objfile);
-      sal = find_pc_line (le[i].pc (objfile), 0);
+      sal = find_sal_for_pc (le[i].pc (objfile), 0);
       mle[newlines].end_pc = sal.end;
       newlines++;
     }
@@ -713,7 +712,7 @@ do_mixed_source_and_assembly (struct gdbarch *gdbarch,
 
   /* The prologue may be empty, but there may still be a line number entry
      for the opening brace which is distinct from the first line of code.
-     If the prologue has been eliminated find_pc_line may return the source
+     If the prologue has been eliminated find_sal_for_pc may return the source
      line after the opening brace.  We still want to print this opening brace.
      first_le is used to implement this.  */
 
@@ -734,7 +733,7 @@ do_mixed_source_and_assembly (struct gdbarch *gdbarch,
       struct symtab_and_line sal;
       int length;
 
-      sal = find_pc_line (pc, 0);
+      sal = find_sal_for_pc (pc, 0);
       length = gdb_insn_length (gdbarch, pc);
       pc += length;
 
@@ -783,7 +782,7 @@ do_mixed_source_and_assembly (struct gdbarch *gdbarch,
       int end_preceding_line_to_display = 0;
       int new_source_line = 0;
 
-      sal = find_pc_line (pc, 0);
+      sal = find_sal_for_pc (pc, 0);
 
       if (sal.symtab != last_symtab)
 	{
@@ -1151,7 +1150,7 @@ gdb_disassembly (struct gdbarch *gdbarch, struct ui_out *uiout,
   int nlines = -1;
 
   /* Assume symtab is valid for whole PC range.  */
-  symtab = find_pc_line_symtab (low);
+  symtab = find_symtab_for_pc (low);
 
   if (symtab != NULL && symtab->linetable () != NULL)
     nlines = symtab->linetable ()->nitems;
@@ -1270,7 +1269,6 @@ set_disassembler_options (const char *prospective_options)
     = make_unique_xstrdup (prospective_options);
   char *options = remove_whitespace_and_extra_commas
     (prospective_options_local.get ());
-  const char *opt;
 
   /* Allow all architectures, even ones that do not support 'set disassembler',
      to reset their disassembler options to NULL.  */
@@ -1292,9 +1290,13 @@ set_disassembler_options (const char *prospective_options)
   valid_options = &valid_options_and_args->options;
 
   /* Verify we have valid disassembler options.  */
-  FOR_EACH_DISASSEMBLER_OPTION (opt, options)
+  char *opt = options;
+  while (1)
     {
       size_t i;
+      char *opt_end = strchr (opt, ',');
+      if (opt_end)
+	*opt_end = 0;
       for (i = 0; valid_options->name[i] != NULL; i++)
 	if (valid_options->arg != NULL && valid_options->arg[i] != NULL)
 	  {
@@ -1309,8 +1311,7 @@ set_disassembler_options (const char *prospective_options)
 	    if (valid_options->arg[i]->values == NULL)
 	      break;
 	    for (j = 0; valid_options->arg[i]->values[j] != NULL; j++)
-	      if (disassembler_options_cmp
-		    (arg, valid_options->arg[i]->values[j]) == 0)
+	      if (strcmp (arg, valid_options->arg[i]->values[j]) == 0)
 		{
 		  found = true;
 		  break;
@@ -1318,7 +1319,7 @@ set_disassembler_options (const char *prospective_options)
 	    if (found)
 	      break;
 	  }
-	else if (disassembler_options_cmp (opt, valid_options->name[i]) == 0)
+	else if (strcmp (opt, valid_options->name[i]) == 0)
 	  break;
       if (valid_options->name[i] == NULL)
 	{
@@ -1327,6 +1328,10 @@ set_disassembler_options (const char *prospective_options)
 		      opt);
 	  return;
 	}
+      if (!opt_end)
+	break;
+      *opt_end = ',';
+      opt = opt_end + 1;
     }
 
   *disassembler_options = options;
@@ -1470,9 +1475,7 @@ disassembler_options_completer (struct cmd_list_element *ignore,
 
 /* Initialization code.  */
 
-void _initialize_disasm ();
-void
-_initialize_disasm ()
+INIT_GDB_FILE (disasm)
 {
   /* Add the command that controls the disassembler options.  */
   set_show_commands set_show_disas_opts
