@@ -1,5 +1,5 @@
 /* LoongArch opcode support.
-   Copyright (C) 2021-2023 Free Software Foundation, Inc.
+   Copyright (C) 2021-2025 Free Software Foundation, Inc.
    Contributed by Loongson Ltd.
 
    This file is part of the GNU opcodes library.
@@ -65,7 +65,17 @@ get_loongarch_opcode_by_binfmt (insn_t insn)
 	if ((insn & it->mask) == it->match && it->mask
 	    && !(it->include && !*it->include)
 	    && !(it->exclude && *it->exclude))
-	  return it;
+	  {
+	    /* ud ui5 need rd==rj. We should continue searching
+	       for the next `it` if rd != rj. Furthermore, we need
+	       `it->pinfo` to ensure that only the `it` in loongarch
+	       alias_opcodes[] is skipped.  */
+	    if (LARCH_INSN_AMSWAP_W (insn)
+		&& (LARCH_GET_RD (insn) != LARCH_GET_RJ (insn))
+		&& (it->pinfo & INSN_DIS_ALIAS))
+	      continue;
+	    return it;
+	  }
     }
   return NULL;
 }
@@ -82,8 +92,8 @@ set_default_loongarch_dis_options (void)
   LARCH_opts.ase_lvz = 1;
   LARCH_opts.ase_lbt = 1;
 
-  loongarch_r_disname = loongarch_r_lp64_name;
-  loongarch_f_disname = loongarch_f_lp64_name;
+  loongarch_r_disname = loongarch_r_alias;
+  loongarch_f_disname = loongarch_f_alias;
   loongarch_fc_disname = loongarch_fc_normal_name;
   loongarch_c_disname = loongarch_c_normal_name;
   loongarch_cr_disname = loongarch_cr_normal_name;
@@ -95,13 +105,18 @@ static int
 parse_loongarch_dis_option (const char *option)
 {
   if (strcmp (option, "no-aliases") == 0)
-    loongarch_dis_show_aliases = false;
+    {
+      loongarch_dis_show_aliases = false;
+      return 0;
+    }
 
   if (strcmp (option, "numeric") == 0)
     {
       loongarch_r_disname = loongarch_r_normal_name;
       loongarch_f_disname = loongarch_f_normal_name;
+      return 0;
     }
+
   return -1;
 }
 
@@ -137,10 +152,19 @@ dis_one_arg (char esc1, char esc2, const char *bit_field,
   insn_t insn = *(insn_t *) info->private_data;
   int32_t imm, u_imm;
   enum disassembler_style style;
+  bool is_ud_2nd_arg = false;
+
+  if (LARCH_INSN_AMSWAP_W (insn)
+      && (LARCH_GET_RD (insn) == LARCH_GET_RJ (insn))
+      && (LARCH_GET_RK (insn) == 1)
+      && loongarch_dis_show_aliases
+      && need_comma)
+    is_ud_2nd_arg = true;
 
   if (esc1)
     {
-      if (need_comma)
+      /* The "ud ui5" does not nedd a comma.  */
+      if (need_comma && !is_ud_2nd_arg)
 	info->fprintf_styled_func (info->stream, dis_style_text, ", ");
       need_comma = 1;
       imm = loongarch_decode_imm (bit_field, insn, 1);
@@ -150,7 +174,17 @@ dis_one_arg (char esc1, char esc2, const char *bit_field,
   switch (esc1)
     {
     case 'r':
-      info->fprintf_styled_func (info->stream, dis_style_register, "%s", loongarch_r_disname[u_imm]);
+      switch (esc2)
+	{
+	  case 'u':
+	    /* The "ud ui5" only needs to print one parameter.  */
+	    if (is_ud_2nd_arg)
+	      break;
+	    info->fprintf_styled_func (info->stream, dis_style_immediate, "0x%x", u_imm);
+	    break;
+	  default:
+	    info->fprintf_styled_func (info->stream, dis_style_register, "%s", loongarch_r_disname[u_imm]);
+	}
       break;
     case 'f':
       switch (esc2)
@@ -267,7 +301,12 @@ disassemble_one (insn_t insn, struct disassemble_info *info)
     }
 
   info->insn_type = dis_nonbranch;
-  info->fprintf_styled_func (info->stream, dis_style_mnemonic, "%-12s", opc->name);
+  if (opc->format == NULL || opc->format[0] == '\0')
+    info->fprintf_styled_func (info->stream, dis_style_mnemonic,
+				"%s", opc->name);
+  else
+    info->fprintf_styled_func (info->stream, dis_style_mnemonic,
+				"%-12s", opc->name);
 
   {
     char *fake_args = xmalloc (strlen (opc->format) + 1);
